from dataclasses import dataclass
import re


@dataclass
class vc():
    key: str
    items: list
    combstring: list


# (till_hour=None, till_minute=None, till_time=None, quadrant=4, duration_hour=0, duration_minute=10, duration_time=None)
class argValidator():
    def __init__(self, args):
        self.till_hour = args.till_hour
        self.till_minute = args.till_minute
        self.till_time = args.till_time
        self.quadrant = args.quadrant
        self.duration_hour = args.duration_hour
        self.duration_minute = args.duration_minute
        self.duration_time = args.duration_time
        self._Combs = {
            "th": vc("th",
                     [self.till_time, self.duration_hour,
                         self.duration_minute, self.duration_time],
                     ["tt", "dh", "dm", "dt"]),

            "tm": vc("tm",
                     [self.till_time, self.duration_hour,
                         self.duration_minute, self.duration_time],
                     ["tt", "dh", "dm", "dt"]),

            "tt": vc("tt",
                     [self.till_hour, self.till_minute, self.duration_hour,
                         self.duration_minute, self.duration_time],
                     ["th", "tm", "dh", "dm", "dt"]),

            "dh": vc("dh",
                     [self.duration_time, self.till_hour,
                         self.till_minute, self.till_time],
                     ["dt", "th", "tm", "tt"]),

            "dm": vc("dm",
                     [self.duration_time, self.till_hour,
                         self.till_minute, self.till_time],
                     ["dt", "th", "tm", "tt"]),

            "dt": vc("dt",
                     [self.duration_hour, self.duration_minute,
                         self.till_hour, self.till_minute, self.till_time],
                     ["dh", "dm", "th", "tm", "tt"])
        }

    def getString(self):
        if self.till_time and any(self._Combs["tt"].items):
            return f"{self._Combs["tt"].key} cannot be used with {self._Combs["tt"].combstring}"

        if self.till_hour and any(self._Combs["th"].items):
            return f"{self._Combs['th'].key} cannot be used with {self._Combs['th'].combstring}"

        if self.till_minute and any(self._Combs["tm"].items):
            return f"{self._Combs['tm'].key} cannot be used with {self._Combs['tm'].combstring}"

        if self.duration_time and any(self._Combs["dt"].items):
            return f"{self._Combs['dt'].key} cannot be used with {self._Combs['dt'].combstring}"

        if self.duration_hour and any(self._Combs["dh"].items):
            return f"{self._Combs['dh'].key} cannot be used with {self._Combs['dh'].combstring}"

        if self.duration_minute and any(self._Combs["dm"].items):
            return f"{self._Combs['dm'].key} cannot be used with {self._Combs['dm'].combstring}"

        return self.dataValidity()

    def dataValidity(self):
        if self.till_time:
            return "please provide tt in valid HH:MM format" if not re.match(r"^(0[0-9]|1[0-9]|2[0-3]):[0-5][0-9]$", self.till_time) else None

        if self.till_hour:
            return "hours must be within 0-23" if self.till_hour > 23 or self.till_hour < 0 else None

        if self.till_minute:
            return "minutes must be within 0-59" if self.till_minute > 59 or self.till_minute < 0 else None

        if self.duration_time:
            return "please provide tt in valid HH:MM format" if not re.match(r"^(0[0-5]):[0-5][0-9]$", self.duration_time) else None

        if self.duration_hour:
            return "hours must be within 0-23" if self.duration_hour > 5 or self.duration_hour < 0 else None

        if self.duration_minute:
            return "minutes must be within 0-59" if self.duration_minute > 59 or self.duration_minute < 0 else None



from argparse import ArgumentParser


parser = ArgumentParser(prog="jigl",
                        description='aumated random mouse movements to avoid defualt sleeps',
                        epilog='Only for ethical purposes'
                        )

parser.add_argument("-th", "--till-hour", type=int,
                    help="sets timer till given hour")

parser.add_argument("-tm", "--till-minute", type=int,
                    help="sets timer till given minute")

parser.add_argument("-tt", "--till-time", type=str,
                    help="sets timer till give hh:mm")

parser.add_argument("-q", "--quadrant", type=int, choices=[1, 2, 3, 4], default=4,
                    help="""sets screen size for the mouse movements
Choices: 1 (Top Left), 2 (Top Right), 3 (Bottom Left), 4 (Bottom Right)
default quadrant is 4""")


parser.add_argument("-dh", "--duration-hour", type=int,
                    help="sets duration for mouse movement in hours Max 5 hrs default 0")

parser.add_argument("-dm", "--duration-minute", type=int,
                    help="sets duration for mouse movement in minutes")

parser.add_argument("-dt", "--duration-time", type=str,
                    help="sets duration for mouse movement in hh:mm")



import pyautogui
from pyautogui import FailSafeException
from datetime import datetime
from datetime import timedelta
import time
import random
from .parserobj import parser
from .validation import argValidator

screenSize = pyautogui.size()
permWidth = screenSize[0]//2
permHieght = screenSize[1]//2


def main():

    args = parser.parse_args()

    if (x := argValidator(args).getString()):
        print(x)
        exit()
    else:
        if (args.till_time):
            args.till_hour = int(args.till_time[:2])
            args.till_minute = int(args.till_time[3:])

        if (args.till_hour or args.till_minute):
            args.till_hour = args.till_hour if args.till_hour else datetime.now().hour
            args.till_minute = args.till_minute if args.till_minute else datetime.now().minute

        if (args.duration_time):
            args.duration_hour = int(args.duration_time[:2])
            args.duration_minute = int(args.duration_time[3:])

        if (args.duration_hour or args.duration_minute):
            args.duration_hour = args.duration_hour if args.duration_hour else 0
            args.duration_minute = args.duration_minute if args.duration_minute else 10

        if not any([args.till_hour, args.till_minute, args.till_time, args.duration_hour, args.duration_minute, args.duration_time]):
            args.duration_hour = 0
            args.duration_minute = 10

        if (any([args.till_time, args.till_hour, args.till_minute]) and (datetime.now() > datetime.now().replace(hour=args.till_hour, minute=args.till_minute))):
            print("timer cant be set into past")
            exit()
        elif (any([args.till_time, args.till_hour, args.till_minute])):
            targetTime = datetime.now().replace(hour=args.till_hour, minute=args.till_minute)
        else:
            targetTime = datetime.now() + timedelta(
                hours=args.duration_hour, minutes=args.duration_minute)

        print(args)
        print(targetTime)

        match args.quadrant:
            case 1:
                offset = [0, 0]
            case 2:
                offset = [permWidth, 0]
            case 3:
                offset = [0, permHieght]
            case 4:
                offset = [permWidth, permHieght]

        # while (datetime.now() != targetTime | pyautogui.position()[]):
        try:
            pyautogui.moveTo(offset[0]+100, offset[1]+100, duration=3)
            pyautogui.click()
            print("screen size ", screenSize)
            print(f"{offset}")
            time.sleep(5)
            pos = pyautogui.position()
            while (datetime.now() < targetTime and not (pos[0] < offset[0] | pos[0] > offset[0] | pos[1] > offset[1] | pos[1] > offset[1])):
                pos = pyautogui.position()
                pyautogui.moveTo(
                    offset[0]+permWidth/2+(permWidth)*random.randint(-5,
                                                                     5)*0.05 + random.randint(-50, 50),
                    offset[1]+permHieght/2+(permHieght)*random.randint(-5,
                                                                       5)*0.03 + random.randint(-30, 30),
                    duration=random.randint(3, 6))
                print(".", end="")
                time.sleep(random.randint(3, 5))
                pyautogui.click()
                print(".", end="")
            print("exiting")
        except FailSafeException:
            print("Edge detected - exiting safely...")
            exit()


if __name__ == "__main__":
    main()
